#include <sragent.h>
#include <srlogger.h>
#include "integrate.h"
#include <cstdlib>
#include <string>
#include <string.h>
#include <srreporter.h>
#include <stdio.h>
#include <iostream>
#include <stdio.h>
#include "clistatus.h"
#include <srutils.h>
#include <time.h>
#include <srdevicepush.h>
#include <unistd.h>
#include "configdb.h"
#include <algorithm>
#include <fstream>
#include <sys/stat.h>
#include <sys/types.h>
#include "digitalOUT.h"
#include "helper.h"
#include <srnetinterface.h>
#include <srnetbinhttp.h>

#define Q(x) ",\"\""#x"\"\""
#define Q2(x) "\"\""#x"\"\""

using namespace std;

const char *ops = ",\"" Q2(c8y_Restart) Q(c8y_RelayArray) Q(c8y_Firmware) Q(c8y_Configuration) Q(c8y_SendConfiguration) Q(c8y_Command) Q(c8y_LogfileRequest) "\"";

const char *logs = ",\"" Q2(syslog) Q(scriptslog) Q(errorlog) "\"";

static SrLogLevel getLogLevel(const string &lvl);

int main()
{
	system("touch /bin/cli");
        system("chmod 755 -R /bin/cli");	
	ConfigDB cdb;

        cdb.load("/etc/c8yAgent.conf");
	const string netBoxIP = cdb.get("netBoxIP");
	if (netBoxIP.empty()) {
		srCritical("No IP.");
		return 0;
	}
	ofstream outputFile("/bin/cli");
	outputFile <<"ssh root@"<< netBoxIP <<" \"/usr/local/sbin/cli $@\"";
	outputFile.close();

	string serialNumber = new_getStatus("SERIAL_NUMBER");
        srLogSetDest(cdb.get("log.path"));
        const uint32_t quota = strtoul(cdb.get("log.quota").c_str(), NULL, 10);
        srLogSetQuota(quota ? quota : 1024); // Default 1024 KB when not set.
        srLogSetLevel(getLogLevel(cdb.get("log.level")));
        const string server = cdb.get("server");
        if (server.empty()) {
                srCritical("No server URL.");
                return 0;
        }

		
	const char *deviceID = serialNumber.c_str();
        srLogSetLevel(SRLOG_DEBUG);        // set log level to debug
        Integrate igt;
        SrAgent agent(server, deviceID, &igt); // instantiate SrAgent
	mkdir(cdb.get("datapath").c_str(), 0750);
        if (agent.bootstrap(cdb.get("datapath") + "credentials")) {
                srCritical("Bootstrap failed.");
                return 0;
        }
	
	string srversion, srtemplate;
        if (readSrTemplate("srtemplate.txt", srversion, srtemplate) != 0)
                return 0;
        if (agent.integrate(srversion, srtemplate)) // integrate to Cumulocity
                return 0;
	string name  = new_getStatus("PRODUCT_NAME");
        string type  = new_getStatus("PRODUCT_TYPE");
        string revision = new_getStatus("HARDWARE_VERSION");
       	agent.send("103," + agent.ID() + "," + type + "," + serialNumber + "," + revision);
      	agent.send("109," + agent.ID() + "," + new_getStatus("OPERATING_SYSTEM") + "," + new_getStatus("SYSTEM_SOFTWARE") + "," + "ftp://share.netmodule.com/router/public/system-software/latest/");
	statusDigitalOut(agent); 	

	SrNetInterface netInterface(server);
	SrNetBinHttp netBinHttp(server, agent.auth());
	
        SrDevicePush push(server, agent.XID(), agent.auth(),
                          agent.ID(), agent.ingress);
	Helper helper(agent, netInterface, netBinHttp, push);
	// Inform Cumulocity about supported operations
	agent.send(SrNews("107," + agent.ID() + ops));
	cout << "-----------------" << endl;	
	// Inform Cumulocity about the logfiles
	agent.send(SrNews("128," + agent.ID() + logs));
	SrReporter reporter(server, agent.XID(), agent.auth(),
                            agent.egress, agent.ingress);
        
	if (reporter.start() || push.start() != 0)      // Start the device push thread
                return 0;
        agent.loop();
        return 0;
}

static SrLogLevel getLogLevel(const string &lvl)
{
        if (lvl == "debug") return SRLOG_DEBUG;
        else if (lvl == "info") return SRLOG_INFO;
        else if (lvl == "notice") return SRLOG_NOTICE;
        else if (lvl == "warning") return SRLOG_WARNING;
        else if (lvl == "error") return SRLOG_ERROR;
        else if (lvl == "critical") return SRLOG_CRITICAL;
        else return SRLOG_INFO;
}

